@@ -1,6 +1,21 @@
 # -*- coding: utf-8 -*-
+'''Wrappers for various units of text, including the main ``TextBlob``,
+``Word``, and ``WordList`` classes.
+Example usage: ::
 
-'''Wrappers for various units of text.'''
+    >>> from text.blob import TextBlob
+    >>> b = TextBlob("Simple is better than complex.")
+    >>> b.tags
+    [(u'Simple', u'NN'), (u'is', u'VBZ'), (u'better', u'JJR'), (u'than', u'IN'), (u'complex', u'NN')]
+    >>> b.noun_phrases
+    WordList([u'simple'])
+    >>> b.words
+    WordList([u'Simple', u'is', u'better', u'than', u'complex'])
+    >>> b.sentiment
+    (0.06666666666666667, 0.41904761904761906)
+    >>> b.words[0].synsets()[0]
+    Synset('simple.n.01')
+'''
 from __future__ import unicode_literals, absolute_import
 import sys
 import json
@@ -8,24 +23,31 @@ import string as pystring
 from collections import defaultdict
 
 from text.packages import nltk
-from text.decorators import cached_property
+from text.decorators import cached_property, requires_nltk_corpus
 from text.utils import lowerstrip, PUNCTUATION_REGEX
 from text.inflect import singularize as _singularize, pluralize as _pluralize
-from text.mixins import BlobComparableMixin
-from text.compat import unicode, basestring, python_2_unicode_compatible, u
-from text.np_extractors import BaseNPExtractor, FastNPExtractor
-from text.taggers import BaseTagger, PatternTagger
-from text.tokenizers import BaseTokenizer, WordTokenizer, SentenceTokenizer
-from text.sentiments import BaseSentimentAnalyzer, PatternAnalyzer
-from text.parsers import BaseParser, PatternParser
+from text.mixins import BlobComparableMixin, StringlikeMixin
+from text.compat import unicode, basestring
+from text.base import (BaseNPExtractor, BaseTagger, BaseTokenizer,
+                       BaseSentimentAnalyzer, BaseParser)
+from text.np_extractors import FastNPExtractor
+from text.taggers import PatternTagger
+from text.tokenizers import WordTokenizer, SentenceTokenizer
+from text.sentiments import PatternAnalyzer
+from text.parsers import PatternParser
 from text.translate import Translator
 from text.en import suggest
-from text.exceptions import MissingCorpusException
+
+# Wordnet interface
+# NOTE: text.wordnet is not imported so that the wordnet corpus can be lazy-loaded
+_wordnet = nltk.corpus.wordnet
 
 
 class Word(unicode):
 
-    '''A simple word representation.'''
+    '''A simple word representation. Includes methods for inflection,
+    translation, and WordNet integration.
+    '''
 
     translator = Translator()
 
@@ -92,14 +114,53 @@ class Word(unicode):
         return Word(self.spellcheck()[0][0])
 
     @cached_property
+    @requires_nltk_corpus
     def lemma(self):
         '''Return the lemma for a word using WordNet's morphy function.'''
         lemmatizer = nltk.stem.WordNetLemmatizer()
-        try:
-            return lemmatizer.lemmatize(self.string)
-        except LookupError as err:
-            print(err)
-            raise MissingCorpusException()
+        return lemmatizer.lemmatize(self.string)
+
+    @cached_property
+    def synsets(self):
+        '''The list of Synset objects for this Word.
+
+        :rtype: list of Synsets
+
+        .. versionadded:: 0.7.0
+        '''
+        return self.get_synsets(pos=None)
+
+    @cached_property
+    def definitions(self):
+        '''The list of definitions for this word. Each definition corresponds
+        to a synset.
+
+        .. versionadded:: 0.7.0
+        '''
+        return self.define(pos=None)
+
+    def get_synsets(self, pos=None):
+        '''Return a list of Synset objects for this word.
+
+        :param pos: A part-of-speech tag to filter upon. If ``None``, all
+            synsets for all parts of speech will be loaded.
+
+        :rtype: list of Synsets
+
+        .. versionadded:: 0.7.0
+        '''
+        return _wordnet.synsets(self.string, pos)
+
+    def define(self, pos=None):
+        '''Return a list of definitions for this word. Each definition
+        corresponds to a synset for this word.
+
+        :param pos: A part-of-speech tag to filter upon. If ``None``, definitions
+            for all parts of speech will be loaded.
+
+        .. versionadded:: 0.7.0
+        '''
+        return [syn.definition for syn in self.get_synsets(pos=pos)]
 
 
 class WordList(list):
@@ -113,6 +174,9 @@ class WordList(list):
         self._collection = [Word(w) for w in collection]
         super(WordList, self).__init__(self._collection)
 
+    def __str__(self):
+        return str(self._collection)
+
     def __repr__(self):
         '''Returns a string representation for debugging.'''
         class_name = self.__class__.__name__
@@ -187,6 +251,7 @@ class WordList(list):
         '''Return the lemma of each word in this WordList.'''
         return self.__class__([word.lemma for word in self])
 
+
 def _validated_param(obj, name, base_class, default, base_class_name=None):
     '''Validates a parameter passed to __init__. Makes sure that obj is
     the correct class. Return obj if it's not None or falls back to default
@@ -199,11 +264,12 @@ def _validated_param(obj, name, base_class, default, base_class_name=None):
     base_class_name = base_class_name if base_class_name else base_class.__name__
     if obj is not None and not isinstance(obj, base_class):
         raise ValueError("{name} must be an instance of {cls}"
-                            .format(name=name, cls=base_class_name))
+                         .format(name=name, cls=base_class_name))
     return obj if obj else default
 
+
 def _initialize_models(obj, tokenizer, pos_tagger,
-                            np_extractor, analyzer, parser, classifier):
+                       np_extractor, analyzer, parser, classifier):
     """Common initialization between BaseBlob and Blobber classes."""
     # tokenizer may be a textblob or an NLTK tokenizer
     obj.tokenizer = _validated_param(tokenizer, "tokenizer",
@@ -220,8 +286,8 @@ def _initialize_models(obj, tokenizer, pos_tagger,
     obj.parser = _validated_param(parser, "parser", BaseParser, BaseBlob.parser)
     obj.classifier = classifier
 
-@python_2_unicode_compatible
-class BaseBlob(BlobComparableMixin):
+
+class BaseBlob(StringlikeMixin, BlobComparableMixin):
 
     '''An abstract base class that all text.blob classes will inherit from.
     Includes words, POS tag, NP, and word count properties. Also includes
@@ -231,13 +297,13 @@ class BaseBlob(BlobComparableMixin):
     :param tokenizer: (optional) A tokenizer instance. If ``None``,
         defaults to :class:`WordTokenizer() <text.tokenizers.WordTokenizer>`.
     :param np_extractor: (optional) An NPExtractor instance. If ``None``,
-        defaults to :class:`FastNPExtractor() <text.np_extractors.FastNPExtractor>`.
+        defaults to :class:`FastNPExtractor() <text.en.np_extractors.FastNPExtractor>`.
     :param pos_tagger: (optional) A Tagger instance. If ``None``,
-        defaults to :class:`PatternTagger <text.taggers.PatternTagger>`.
+        defaults to :class:`PatternTagger <text.en.taggers.PatternTagger>`.
     :param analyzer: (optional) A sentiment analyzer. If ``None``,
-        defaults to :class:`PatternAnalyzer <text.sentiments.PatternAnalyzer>`.
+        defaults to :class:`PatternAnalyzer <text.en.sentiments.PatternAnalyzer>`.
     :param parser: A parser. If ``None``, defaults to
-        :class:`PatternParser <text.parsers.PatternParser>`.
+        :class:`PatternParser <text.en.parsers.PatternParser>`.
     :param classifier: A classifier.
 
     .. versionchanged:: 0.6.0
@@ -263,7 +329,8 @@ class BaseBlob(BlobComparableMixin):
                                     "get_text() function")
         self.raw = self.string = text
         self.stripped = lowerstrip(self.raw, all=True)
-        _initialize_models(self, tokenizer, pos_tagger, np_extractor, analyzer, parser, classifier)
+        _initialize_models(self, tokenizer, pos_tagger, np_extractor, analyzer,
+                           parser, classifier)
 
     @cached_property
     def words(self):
@@ -303,7 +370,7 @@ class BaseBlob(BlobComparableMixin):
     def classify(self):
         '''Classify the blob using the blob's ``classifier``.'''
         if self.classifier is None:
-            raise NameError("This blob has no classfier. Train one first!")
+            raise NameError("This blob has no classifier. Train one first!")
         return self.classifier.classify(self.raw)
 
     @cached_property
@@ -458,50 +525,19 @@ class BaseBlob(BlobComparableMixin):
                 ret = ' '.join([ret, word])
         return self.__class__(ret)
 
-    def __repr__(self):
-        '''Returns a string representation for debugging.'''
-        class_name = self.__class__.__name__
-        return "{cls}({text})".format(cls=class_name,
-                                        text=repr(self.raw))
-    def __len__(self):
-        '''Returns the length of the raw text.'''
-        return len(self.raw)
-
-    def __str__(self):
-        '''Returns a string representation used in print statements
-        or str(my_blob).'''
-        return self.raw
-
-    def __unicode__(self):
-        '''Returns the unicode representation of the blob.'''
-        return u(self.raw)
-
-    def __iter__(self):
-        '''Makes the object iterable as if it were a string,
-        iterating through the raw string's characters.
-        '''
-        return iter(self.raw)
-
     def _cmpkey(self):
         '''Key used by ComparableMixin to implement all rich comparison
         operators.
         '''
         return self.raw
 
+    def _strkey(self):
+        '''Key used by StringlikeMixin to implement string methods.'''
+        return self.raw
+
     def __hash__(self):
         return hash(self._cmpkey())
 
-    def __getitem__(self, index):
-        '''Returns a  substring. If index is an integer, returns a Python
-        string of a single character. If a range is given, e.g. `blob[3:5]`,
-        a new instance of the class is returned.
-        '''
-        if isinstance(index, int):
-            return self.raw[index]  # Just return a single character
-        else:
-            # Return a new blob object
-            return self.__class__(self.raw[index])
-
     def __add__(self, other):
         '''Concatenates two text objects the same way Python strings are
         concatenated.
@@ -510,95 +546,18 @@ class BaseBlob(BlobComparableMixin):
         - `other`: a string or a text object
         '''
         if isinstance(other, basestring):
-            return TextBlob(self.raw + other)
+            return self.__class__(self.raw + other)
         elif isinstance(other, BaseBlob):
-            return TextBlob(self.raw + other.raw)
+            return self.__class__(self.raw + other.raw)
         else:
             raise TypeError('Operands must be either strings or {0} objects'
                 .format(self.__class__.__name__))
 
-    def __contains__(self, sub):
-        '''Implements the `in` keyword like a Python string.'''
-        return sub in self.raw
-
-    def find(self, sub, start=0, end=sys.maxsize):
-        '''Behaves like the built-in str.find() method. Returns an integer,
-        the index of the first occurrence of the substring argument sub in the
-        sub-string given by [start:end].
-        '''
-        return self.raw.find(sub, start, end)
-
-    def rfind(self, sub, start=0, end=sys.maxsize):
-        '''Behaves like the built-in str.rfind() method. Returns an integer,
-        the index of he last (right-most) occurence of the substring argument
-        sub in the sub-sequence given by [start:end].
-        '''
-        return self.raw.rfind(sub, start, end)
-
-    def index(self, sub, start=0, end=sys.maxsize):
-        '''Like blob.find() but raise ValueError when the substring
-        is not found.
-        '''
-        return self.raw.index(sub, start, end)
-
-    def startswith(self, prefix, start=0, end=sys.maxsize):
-        """Returns True if the blob starts with the given prefix."""
-        return self.raw.startswith(prefix, start, end)
-
-    def endswith(self, suffix, start=0, end=sys.maxsize):
-        """Returns True if the blob ends with the given suffix."""
-        return self.raw.endswith(suffix, start, end)
-
-    # PEP8 aliases
-    starts_with = startswith
-    ends_with = endswith
-
-    def title(self):
-        """Returns a blob object with the text in title-case."""
-        return self.__class__(self.raw.title())
-
-    def format(self, *args, **kwargs):
-        """Perform a string formatting operation, like the built-in
-        `str.format(*args, **kwargs)`. Returns a blob object.
-        """
-        return self.__class__(self.raw.format(*args, **kwargs))
-
     def split(self, sep=None, maxsplit=sys.maxsize):
         """Behaves like the built-in str.split() except returns a
         WordList.
         """
-        return WordList(self.raw.split(sep, maxsplit))
-
-    def strip(self, chars=None):
-        """Behaves like the built-in str.strip([chars]) method. Returns
-        an object with leading and trailing whitespace removed.
-        """
-        return self.__class__(self.raw.strip(chars))
-
-    def upper(self):
-        """Like str.upper(), returns new object with all upper-cased characters.
-        """
-        return self.__class__(self.raw.upper())
-
-    def lower(self):
-        """Like str.lower(), returns new object with all lower-cased characters.
-        """
-        return self.__class__(self.raw.lower())
-
-    def join(self, iterable):
-        """Behaves like the built-in `str.join(iterable)` method, except
-        returns a blob object.
-
-        Returns a blob which is the concatenation of the strings or blobs
-        in the iterable.
-        """
-        return self.__class__(self.raw.join(iterable))
-
-    def replace(self, old, new, count=sys.maxsize):
-        """Return a new blob object with all the occurence of `old` replaced
-        by `new`.
-        """
-        return self.__class__(self.raw.replace(old, new, count))
+        return WordList(self._strkey().split(sep, maxsplit))
 
 
 class TextBlob(BaseBlob):
@@ -610,11 +569,11 @@ class TextBlob(BaseBlob):
     :param tokenizer: (optional) A tokenizer instance. If ``None``, defaults to
         :class:`WordTokenizer() <text.tokenizers.WordTokenizer>`.
     :param np_extractor: (optional) An NPExtractor instance. If ``None``,
-        defaults to :class:`FastNPExtractor() <text.np_extractors.FastNPExtractor>`.
+        defaults to :class:`FastNPExtractor() <text.en.np_extractors.FastNPExtractor>`.
     :param pos_tagger: (optional) A Tagger instance. If ``None``, defaults to
-        :class:`PatternTagger <text.taggers.PatternTagger>`.
+        :class:`PatternTagger <text.en.taggers.PatternTagger>`.
     :param analyzer: (optional) A sentiment analyzer. If ``None``, defaults to
-        :class:`PatternAnalyzer <text.sentiments.PatternAnalyzer>`.
+        :class:`PatternAnalyzer <text.en.sentiments.PatternAnalyzer>`.
     :param classifier: (optional) A classifier.
     """
 
@@ -764,12 +723,13 @@ class Blobber(object):
 
     def __call__(self, text):
         '''Return a new TextBlob object with this Blobber's ``np_extractor``,
-        ``pos_tagger``, and ``tokenizer``.
+        ``pos_tagger``, ``tokenizer``, ``analyzer``, and ``classifier``.
 
         :returns: A new TextBlob.
         '''
         return TextBlob(text, tokenizer=self.tokenizer, pos_tagger=self.pos_tagger,
                         np_extractor=self.np_extractor, analyzer=self.analyzer,
+                        parser=self.parser,
                         classifier=self.classifier)
 
     def __repr__(self):
